@@ -3,30 +3,36 @@
 #ifdef FF_HAVE_XRANDR
 
 #include "common/library.h"
-#include "common/parsing.h"
-#include "util/edidHelper.h"
-#include "util/stringUtils.h"
+#include "common/properties.h"
+#include "common/edidHelper.h"
+#include "common/stringUtils.h"
 
 #include <X11/extensions/Xrandr.h>
 #include <X11/Xlib.h>
 
-typedef struct X11PropertyData
+typedef struct XrandrData
 {
     FF_LIBRARY_SYMBOL(XInternAtom)
+    FF_LIBRARY_SYMBOL(XGetAtomName)
     FF_LIBRARY_SYMBOL(XGetWindowProperty)
+    FF_LIBRARY_SYMBOL(XServerVendor)
     FF_LIBRARY_SYMBOL(XFree)
-} X11PropertyData;
+    FF_LIBRARY_SYMBOL(XRRGetMonitors)
+    FF_LIBRARY_SYMBOL(XRRGetScreenResourcesCurrent)
+    FF_LIBRARY_SYMBOL(XRRGetOutputInfo)
+    FF_LIBRARY_SYMBOL(XRRGetOutputProperty)
+    FF_LIBRARY_SYMBOL(XRRGetCrtcInfo)
+    FF_LIBRARY_SYMBOL(XRRFreeCrtcInfo)
+    FF_LIBRARY_SYMBOL(XRRFreeOutputInfo)
+    FF_LIBRARY_SYMBOL(XRRFreeScreenResources)
+    FF_LIBRARY_SYMBOL(XRRFreeMonitors)
 
-static bool x11InitPropertyData(FF_MAYBE_UNUSED void* libraryHandle, X11PropertyData* propertyData)
-{
-    FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XInternAtom, false)
-    FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XGetWindowProperty, false)
-    FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XFree, false)
+    //Init once
+    Display* display;
+    FFDisplayServerResult* result;
+} XrandrData;
 
-    return true;
-}
-
-static unsigned char* x11GetProperty(X11PropertyData* data, Display* display, Window window, const char* request)
+static unsigned char* x11GetProperty(XrandrData* data, Display* display, Window window, const char* request)
 {
     Atom requestAtom = data->ffXInternAtom(display, request, False);
     if(requestAtom == None)
@@ -42,18 +48,18 @@ static unsigned char* x11GetProperty(X11PropertyData* data, Display* display, Wi
     return result;
 }
 
-static void x11DetectWMFromEWMH(X11PropertyData* data, Display* display, FFDisplayServerResult* result)
+static void x11DetectWMFromEWMH(XrandrData* data, FFDisplayServerResult* result)
 {
     if(result->wmProcessName.length > 0 || ffStrbufCompS(&result->wmProtocolName, FF_WM_PROTOCOL_WAYLAND) == 0)
         return;
 
-    Window* wmWindow = (Window*) x11GetProperty(data, display, DefaultRootWindow(display), "_NET_SUPPORTING_WM_CHECK");
+    Window* wmWindow = (Window*) x11GetProperty(data, data->display, DefaultRootWindow(data->display), "_NET_SUPPORTING_WM_CHECK");
     if(wmWindow == NULL)
         return;
 
-    char* wmName = (char*) x11GetProperty(data, display, *wmWindow, "_NET_WM_NAME");
-    if(wmName == NULL)
-        wmName = (char*) x11GetProperty(data, display, *wmWindow, "WM_NAME");
+    char* wmName = (char*) x11GetProperty(data, data->display, *wmWindow, "WM_NAME");
+    if(!ffStrSet(wmName))
+        wmName = (char*) x11GetProperty(data, data->display, *wmWindow, "_NET_WM_NAME");
 
     if(ffStrSet(wmName))
         ffStrbufSetS(&result->wmProcessName, wmName);
@@ -62,36 +68,20 @@ static void x11DetectWMFromEWMH(X11PropertyData* data, Display* display, FFDispl
     data->ffXFree(wmWindow);
 }
 
-typedef struct XrandrData
+static void x11FetchServerVendor(XrandrData* data, FFDisplayServerResult* result)
 {
-    FF_LIBRARY_SYMBOL(XInternAtom)
-    FF_LIBRARY_SYMBOL(XGetAtomName);
-    FF_LIBRARY_SYMBOL(XFree);
-    FF_LIBRARY_SYMBOL(XRRGetMonitors)
-    FF_LIBRARY_SYMBOL(XRRGetScreenResourcesCurrent)
-    FF_LIBRARY_SYMBOL(XRRGetOutputInfo)
-    FF_LIBRARY_SYMBOL(XRRGetOutputProperty)
-    FF_LIBRARY_SYMBOL(XRRGetCrtcInfo)
-    FF_LIBRARY_SYMBOL(XRRFreeCrtcInfo)
-    FF_LIBRARY_SYMBOL(XRRFreeOutputInfo)
-    FF_LIBRARY_SYMBOL(XRRFreeScreenResources)
-    FF_LIBRARY_SYMBOL(XRRFreeMonitors)
+    const char* serverVendor = data->ffXServerVendor(data->display);
+    if (serverVendor && !ffStrEquals(serverVendor, "The X.Org Foundation"))
+        ffStrbufSetS(&result->wmProtocolName, serverVendor);
+}
 
-    //Init once
-    Display* display;
-    FFDisplayServerResult* result;
-
-    //Init per screen
-    XRRScreenResources* screenResources;
-} XrandrData;
-
-static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf* name, bool primary, FFDisplayType displayType, uint8_t* edidData, uint32_t edidLength)
+static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf* name, bool primary, FFDisplayType displayType, uint8_t* edidData, uint32_t edidLength, XRRScreenResources* screenResources, uint8_t bitDepth, double scaleFactor)
 {
     //We do the check here, because we want the best fallback display if this call failed
-    if(data->screenResources == NULL)
+    if(screenResources == NULL)
         return false;
 
-    XRRCrtcInfo* crtcInfo = data->ffXRRGetCrtcInfo(data->display, data->screenResources, output->crtc);
+    XRRCrtcInfo* crtcInfo = data->ffXRRGetCrtcInfo(data->display, screenResources, output->crtc);
     if(crtcInfo == NULL)
         return false;
 
@@ -113,27 +103,24 @@ static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf*
     }
 
     XRRModeInfo* currentMode = NULL;
-    if (data->screenResources)
+    for(int i = 0; i < screenResources->nmode; i++)
     {
-        for(int i = 0; i < data->screenResources->nmode; i++)
+        if(screenResources->modes[i].id == crtcInfo->mode)
         {
-            if(data->screenResources->modes[i].id == crtcInfo->mode)
-            {
-                currentMode = &data->screenResources->modes[i];
-                break;
-            }
+            currentMode = &screenResources->modes[i];
+            break;
         }
     }
 
-    XRRModeInfo* preferredMode = data->screenResources && output->npreferred > 0 ? &data->screenResources->modes[0] : NULL;
+    XRRModeInfo* preferredMode = output->npreferred > 0 ? &screenResources->modes[0] : NULL;
 
     FFDisplayResult* item = ffdsAppendDisplay(
         data->result,
-        (uint32_t) crtcInfo->width,
-        (uint32_t) crtcInfo->height,
+        (uint32_t) (currentMode ? currentMode->width : crtcInfo->width),
+        (uint32_t) (currentMode ? currentMode->height : crtcInfo->height),
         currentMode ? (double) currentMode->dotClock / (double) ((uint32_t) currentMode->hTotal * currentMode->vTotal) : 0,
-        (uint32_t) crtcInfo->width,
-        (uint32_t) crtcInfo->height,
+        (uint32_t) (crtcInfo->width / scaleFactor + .5),
+        (uint32_t) (crtcInfo->height / scaleFactor + .5),
         preferredMode ? (uint32_t) preferredMode->width : 0,
         preferredMode ? (uint32_t) preferredMode->height : 0,
         preferredMode ? (double) preferredMode->dotClock / (double) ((uint32_t) preferredMode->hTotal * preferredMode->vTotal) : 0,
@@ -144,22 +131,26 @@ static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf*
         0,
         (uint32_t) output->mm_width,
         (uint32_t) output->mm_height,
-        "xlib-randr-crtc"
+        currentMode ? "xlib-randr-mode" : "xlib-randr-crtc"
     );
 
-    if (item && edidLength)
+    if (item)
     {
-        item->hdrStatus = ffEdidGetHdrCompatible(edidData, edidLength) ? FF_DISPLAY_HDR_STATUS_SUPPORTED : FF_DISPLAY_HDR_STATUS_UNSUPPORTED;
-        ffEdidGetSerialAndManufactureDate(edidData, &item->serial, &item->manufactureYear, &item->manufactureWeek);
+        if (edidLength)
+        {
+            item->hdrStatus = ffEdidGetHdrCompatible(edidData, edidLength) ? FF_DISPLAY_HDR_STATUS_SUPPORTED : FF_DISPLAY_HDR_STATUS_UNSUPPORTED;
+            ffEdidGetSerialAndManufactureDate(edidData, &item->serial, &item->manufactureYear, &item->manufactureWeek);
+        }
+        item->bitDepth = bitDepth;
     }
 
     data->ffXRRFreeCrtcInfo(crtcInfo);
     return !!item;
 }
 
-static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name, bool primary, FFDisplayType displayType)
+static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name, bool primary, FFDisplayType displayType, XRRScreenResources* screenResources, uint8_t bitDepth, double scaleFactor)
 {
-    XRROutputInfo* outputInfo = data->ffXRRGetOutputInfo(data->display, data->screenResources, output);
+    XRROutputInfo* outputInfo = data->ffXRRGetOutputInfo(data->display, screenResources, output);
     if(outputInfo == NULL)
         return false;
 
@@ -183,7 +174,7 @@ static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name
         }
     }
 
-    bool res = xrandrHandleCrtc(data, outputInfo, name, primary, displayType, edidData, (uint32_t) edidLength);
+    bool res = xrandrHandleCrtc(data, outputInfo, name, primary, displayType, edidData, (uint32_t) edidLength, screenResources, bitDepth, scaleFactor);
 
     if (edidData)
         data->ffXFree(edidData);
@@ -192,7 +183,7 @@ static bool xrandrHandleOutput(XrandrData* data, RROutput output, FFstrbuf* name
     return res;
 }
 
-static bool xrandrHandleMonitor(XrandrData* data, XRRMonitorInfo* monitorInfo)
+static bool xrandrHandleMonitor(XrandrData* data, XRRMonitorInfo* monitorInfo, XRRScreenResources* screenResources, uint8_t bitDepth, double scaleFactor)
 {
     bool foundOutput = false;
     char* xname = data->ffXGetAtomName(data->display, monitorInfo->name);
@@ -201,17 +192,19 @@ static bool xrandrHandleMonitor(XrandrData* data, XRRMonitorInfo* monitorInfo)
     FFDisplayType displayType = ffdsGetDisplayType(name.chars);
     for(int i = 0; i < monitorInfo->noutput; i++)
     {
-        if(xrandrHandleOutput(data, monitorInfo->outputs[i], &name, monitorInfo->primary, displayType))
+        if(xrandrHandleOutput(data, monitorInfo->outputs[i], &name, monitorInfo->primary, displayType, screenResources, bitDepth, scaleFactor))
             foundOutput = true;
     }
 
-    return foundOutput ? true : !!ffdsAppendDisplay(
+    if (foundOutput) return true;
+
+    FFDisplayResult* display = ffdsAppendDisplay(
         data->result,
         (uint32_t) monitorInfo->width,
         (uint32_t) monitorInfo->height,
         0,
-        (uint32_t) monitorInfo->width,
-        (uint32_t) monitorInfo->height,
+        (uint32_t) (monitorInfo->width / scaleFactor + .5),
+        (uint32_t) (monitorInfo->height / scaleFactor + .5),
         0, 0, 0,
         0,
         &name,
@@ -222,6 +215,8 @@ static bool xrandrHandleMonitor(XrandrData* data, XRRMonitorInfo* monitorInfo)
         (uint32_t) monitorInfo->mheight,
         "xlib-randr-monitor"
     );
+    if (display) display->bitDepth = bitDepth;
+    return !!display;
 }
 
 static bool xrandrHandleMonitors(XrandrData* data, Screen* screen)
@@ -231,29 +226,36 @@ static bool xrandrHandleMonitors(XrandrData* data, Screen* screen)
     if(monitorInfos == NULL)
         return false;
 
+    XRRScreenResources* screenResources = data->ffXRRGetScreenResourcesCurrent(data->display, RootWindowOfScreen(screen));
+
+    double scaleFactor = 1;
+    char* resourceManager = (char*) x11GetProperty(data, data->display, screen->root, "RESOURCE_MANAGER");
+    if (resourceManager)
+    {
+        FF_STRBUF_AUTO_DESTROY dpi = ffStrbufCreate();
+        if (ffParsePropLines(resourceManager, "Xft.dpi:", &dpi))
+            scaleFactor = ffStrbufToDouble(&dpi, 96) / 96;
+        data->ffXFree(resourceManager);
+    }
+    uint8_t bitDepth = (uint8_t) (screen->root_depth / 3);
+
     bool foundAMonitor = false;
 
     for(int i = 0; i < numberOfMonitors; i++)
     {
-        if(xrandrHandleMonitor(data, &monitorInfos[i]))
+        if(xrandrHandleMonitor(data, &monitorInfos[i], screenResources, bitDepth, scaleFactor))
             foundAMonitor = true;
     }
 
     data->ffXRRFreeMonitors(monitorInfos);
+    data->ffXRRFreeScreenResources(screenResources);
 
     return foundAMonitor;
 }
 
 static void xrandrHandleScreen(XrandrData* data, Screen* screen)
 {
-    //Init screen resources
-    data->screenResources = data->ffXRRGetScreenResourcesCurrent(data->display, RootWindowOfScreen(screen));
-
-    bool ret = xrandrHandleMonitors(data, screen);
-
-    data->ffXRRFreeScreenResources(data->screenResources);
-
-    if(ret)
+    if(xrandrHandleMonitors(data, screen))
         return;
 
     //Fallback to screen
@@ -269,7 +271,7 @@ static void xrandrHandleScreen(XrandrData* data, Screen* screen)
         NULL,
         FF_DISPLAY_TYPE_UNKNOWN,
         false,
-        0,
+        RootWindowOfScreen(screen),
         (uint32_t) WidthMMOfScreen(screen),
         (uint32_t) HeightMMOfScreen(screen),
         "xlib-randr-screen"
@@ -278,7 +280,7 @@ static void xrandrHandleScreen(XrandrData* data, Screen* screen)
 
 const char* ffdsConnectXrandr(FFDisplayServerResult* result)
 {
-    FF_LIBRARY_LOAD(xrandr, "dlopen libXrandr failed", "libXrandr" FF_LIBRARY_EXTENSION, 3)
+    FF_LIBRARY_LOAD_MESSAGE(xrandr, "libXrandr" FF_LIBRARY_EXTENSION, 3)
 
     FF_LIBRARY_LOAD_SYMBOL_MESSAGE(xrandr, XOpenDisplay)
     FF_LIBRARY_LOAD_SYMBOL_MESSAGE(xrandr, XCloseDisplay)
@@ -287,6 +289,8 @@ const char* ffdsConnectXrandr(FFDisplayServerResult* result)
 
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XInternAtom);
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XGetAtomName);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XGetWindowProperty);
+    FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XServerVendor);
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XFree);
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetMonitors);
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRGetScreenResourcesCurrent);
@@ -298,15 +302,14 @@ const char* ffdsConnectXrandr(FFDisplayServerResult* result)
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeScreenResources);
     FF_LIBRARY_LOAD_SYMBOL_VAR_MESSAGE(xrandr, data, XRRFreeMonitors);
 
-    X11PropertyData propertyData;
-    bool propertyDataInitialized = x11InitPropertyData(xrandr, &propertyData);
-
     data.display = ffXOpenDisplay(NULL);
     if(data.display == NULL)
         return "XOpenDisplay() failed";
 
-    if(propertyDataInitialized && ScreenCount(data.display) > 0)
-        x11DetectWMFromEWMH(&propertyData, data.display, result);
+    if(ScreenCount(data.display) > 0) {
+        x11DetectWMFromEWMH(&data, result);
+        x11FetchServerVendor(&data, result);
+    }
 
     data.result = result;
 
