@@ -1,8 +1,8 @@
 #include "common/printing.h"
 #include "common/jsonconfig.h"
+#include "common/stringUtils.h"
 #include "detection/weather/weather.h"
 #include "modules/weather/weather.h"
-#include "util/stringUtils.h"
 
 bool ffPrintWeather(FFWeatherOptions* options)
 {
